@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 """
 cluster.py: prototyping/experimentation for distributed Mininet,
@@ -74,16 +74,6 @@ Things to do:
 - hifi support (e.g. delay compensation)
 """
 
-from mininet.node import Node, Host, OVSSwitch, Controller
-from mininet.link import Link, Intf
-from mininet.net import Mininet
-from mininet.topo import LinearTopo
-from mininet.topolib import TreeTopo
-from mininet.util import quietRun, errRun
-from mininet.examples.clustercli import CLI
-from mininet.log import setLogLevel, debug, info, error
-from mininet.clean import addCleanupCallback
-
 from signal import signal, SIGINT, SIG_IGN
 from subprocess import Popen, PIPE, STDOUT
 import os
@@ -94,6 +84,18 @@ from itertools import groupby
 from operator import attrgetter
 from distutils.version import StrictVersion
 
+from mininet.node import Node, Host, OVSSwitch, Controller
+from mininet.link import Link, Intf
+from mininet.net import Mininet
+from mininet.topo import LinearTopo
+from mininet.topolib import TreeTopo
+from mininet.util import quietRun, errRun, decode
+from mininet.examples.clustercli import CLI
+from mininet.log import setLogLevel, debug, info, error
+from mininet.clean import addCleanupCallback
+
+# pylint: disable=too-many-arguments
+
 
 def findUser():
     "Try to return logged-in (usually non-root) user"
@@ -125,7 +127,7 @@ class ClusterCleanup( object ):
     def cleanup( cls ):
         "Clean up"
         info( '*** Cleaning up cluster\n' )
-        for server, user in cls.serveruser.iteritems():
+        for server, user in cls.serveruser.items():
             if server == 'localhost':
                 # Handled by mininet.clean.cleanup()
                 continue
@@ -238,14 +240,14 @@ class RemoteMixin( object ):
            args: string or list of strings
            returns: stdout and stderr"""
         popen = self.rpopen( *cmd, **opts )
-        # print 'RCMD: POPEN:', popen
+        # info( 'RCMD: POPEN:', popen, '\n' )
         # These loops are tricky to get right.
         # Once the process exits, we can read
         # EOF twice if necessary.
         result = ''
         while True:
             poll = popen.poll()
-            result += popen.stdout.read()
+            result += decode( popen.stdout.read() )
             if poll is not None:
                 break
         return result
@@ -260,7 +262,7 @@ class RemoteMixin( object ):
             cmd: remote command to run (list)
             **params: parameters to Popen()
             returns: Popen() object"""
-        if type( cmd ) is str:
+        if isinstance( cmd, str):
             cmd = cmd.split()
         if self.isRemote:
             if sudo:
@@ -287,7 +289,8 @@ class RemoteMixin( object ):
 
     def addIntf( self, *args, **kwargs ):
         "Override: use RemoteLink.moveIntf"
-        kwargs.update( moveIntfFn=RemoteLink.moveIntf )
+        # kwargs.update( moveIntfFn=RemoteLink.moveIntf )
+        # pylint: disable=useless-super-delegation
         return super( RemoteMixin, self).addIntf( *args, **kwargs )
 
 
@@ -324,6 +327,7 @@ class RemoteOVSSwitch( RemoteMixin, OVSSwitch ):
                  StrictVersion( '1.10' ) )
 
     @classmethod
+    # pylint: disable=arguments-differ
     def batchStartup( cls, switches, **_kwargs ):
         "Start up switches in per-server batches"
         key = attrgetter( 'server' )
@@ -335,6 +339,7 @@ class RemoteOVSSwitch( RemoteMixin, OVSSwitch ):
         return switches
 
     @classmethod
+    # pylint: disable=arguments-differ
     def batchShutdown( cls, switches, **_kwargs ):
         "Stop switches in per-server batches"
         key = attrgetter( 'server' )
@@ -392,33 +397,31 @@ class RemoteLink( Link ):
         return self.tunnel
 
     @staticmethod
-    def moveIntf( intf, node, printError=True ):
+    def moveIntf( intf, node ):
         """Move remote interface from root ns to node
             intf: string, interface
             dstNode: destination Node
-            srcNode: source Node or None (default) for root ns
-            printError: if true, print error"""
+            srcNode: source Node or None (default) for root ns"""
         intf = str( intf )
         cmd = 'ip link set %s netns %s' % ( intf, node.pid )
-        node.rcmd( cmd )
-        links = node.cmd( 'ip link show' )
-        if not re.search( r' %s[:@]' % intf, links ):
-            if printError:
-                error( '*** Error: RemoteLink.moveIntf: ' + intf +
-                       ' not successfully moved to ' + node.name + '\n' )
-            return False
+        result = node.rcmd( cmd )
+        if result:
+            raise Exception('error executing command %s' % cmd)
         return True
 
     def makeTunnel( self, node1, node2, intfname1, intfname2,
                     addr1=None, addr2=None ):
         "Make a tunnel across switches on different servers"
         # We should never try to create a tunnel to ourselves!
-        assert node1.server != 'localhost' or node2.server != 'localhost'
+        assert node1.server != node2.server
         # And we can't ssh into this server remotely as 'localhost',
         # so try again swappping node1 and node2
         if node2.server == 'localhost':
-            return self.makeTunnel( node2, node1, intfname2, intfname1,
-                                    addr2, addr1 )
+            return self.makeTunnel( node1=node2, node2=node1,
+                                    intfname1=intfname2, intfname2=intfname1,
+                                    addr1=addr2, addr2=addr1 )
+        debug( '\n*** Make SSH tunnel ' + node1.server + ':' + intfname1 +
+               ' == ' + node2.server + ':' + intfname2 )
         # 1. Create tap interfaces
         for node in node1, node2:
             # For now we are hard-wiring tap9, which we will rename
@@ -437,13 +440,16 @@ class RemoteLink( Link ):
         # When we receive the character '@', it means that our
         # tunnel should be set up
         debug( 'Waiting for tunnel to come up...\n' )
-        ch = tunnel.stdout.read( 1 )
+        ch = decode( tunnel.stdout.read( 1 ) )
         if ch != '@':
-            raise Exception( 'makeTunnel:\n',
-                             'Tunnel setup failed for',
-                             '%s:%s' % ( node1, node1.dest ), 'to',
-                             '%s:%s\n' % ( node2, node2.dest ),
-                             'command was:', cmd, '\n' )
+            ch += decode( tunnel.stdout.read() )
+            cmd = ' '.join( cmd )
+            raise Exception( 'makeTunnel:\n'
+                             'Tunnel setup failed for '
+                             '%s:%s' % ( node1, node1.dest ) + ' to '
+                             '%s:%s\n' % ( node2, node2.dest ) +
+                             'command was: %s' % cmd + '\n' +
+                             'result was: ' + ch )
         # 3. Move interfaces if necessary
         for node in node1, node2:
             if not self.moveIntf( 'tap9', node ):
@@ -474,6 +480,93 @@ class RemoteLink( Link ):
         return result
 
 
+class RemoteSSHLink( RemoteLink ):
+    "Remote link using SSH tunnels"
+    def __init__(self, node1, node2, **kwargs):
+        RemoteLink.__init__( self, node1, node2, **kwargs )
+
+
+class RemoteGRELink( RemoteLink ):
+    "Remote link using GRE tunnels"
+
+    GRE_KEY = 0
+
+    def __init__(self, node1, node2, **kwargs):
+        RemoteLink.__init__( self, node1, node2, **kwargs )
+
+    def stop( self ):
+        "Stop this link"
+        if self.tunnel:
+            self.intf1.delete()
+            self.intf2.delete()
+        else:
+            Link.stop( self )
+        self.tunnel = None
+
+    def makeIntfPair( self, intfname1, intfname2, addr1=None, addr2=None,
+                      node1=None, node2=None, deleteIntfs=True   ):
+        """Create pair of interfaces
+            intfname1: name of interface 1
+            intfname2: name of interface 2
+            (override this method [and possibly delete()]
+            to change link type)"""
+        node1 = self.node1 if node1 is None else node1
+        node2 = self.node2 if node2 is None else node2
+        server1 = getattr( node1, 'server', 'localhost' )
+        server2 = getattr( node2, 'server', 'localhost' )
+        if server1 == server2:
+            # Link within same server
+            Link.makeIntfPair( intfname1, intfname2, addr1, addr2,
+                               node1, node2, deleteIntfs=deleteIntfs )
+            # Need to reduce the MTU of all emulated hosts to 1450 for GRE
+            # tunneling, otherwise packets larger than 1400 bytes cannot be
+            # successfully transmitted through the tunnel.
+            node1.cmd('ip link set dev %s mtu 1450' % intfname1)
+            node2.cmd('ip link set dev %s mtu 1450' % intfname2)
+        else:
+            # Otherwise, make a tunnel
+            self.makeTunnel( node1, node2, intfname1, intfname2, addr1, addr2 )
+            self.tunnel = 1
+
+    def makeTunnel(self, node1, node2, intfname1, intfname2,
+                       addr1=None, addr2=None):
+        "Make a tunnel across switches on different servers"
+        # We should never try to create a tunnel to ourselves!
+        assert node1.server != node2.server
+        if node2.server == 'localhost':
+            return self.makeTunnel( node1=node2, node2=node1,
+                                    intfname1=intfname2, intfname2=intfname1,
+                                    addr1=addr2, addr2=addr1 )
+        IP1, IP2 = node1.serverIP, node2.serverIP
+        # GRE tunnel needs to be set up with the IP of the local interface
+        # that connects the remote node, NOT '127.0.0.1' of localhost
+        if node1.server == 'localhost':
+            output = quietRun('ip route get %s' % node2.serverIP)
+            IP1 = output.split(' src ')[1].split()[0]
+        debug( '\n*** Make GRE tunnel ' + node1.server + ':' + intfname1 +
+               ' == ' + node2.server + ':' + intfname2 )
+        tun1 = 'local ' + IP1 + ' remote ' + IP2
+        tun2 = 'local ' + IP2 + ' remote ' + IP1
+        self.__class__.GRE_KEY += 1
+        for (node, intfname, addr, tun) in [(node1, intfname1, addr1, tun1),
+                                            (node2, intfname2, addr2, tun2)]:
+            node.rcmd('ip link delete ' + intfname)
+            result = node.rcmd('ip link add name ' + intfname + ' type gretap '
+                               + tun + ' ttl 64 key '
+                               + str( self.__class__.GRE_KEY) )
+            if result:
+                raise Exception('error creating gretap on %s: %s'
+                                % (node, result))
+            if addr:
+                node.rcmd('ip link set %s address %s' % (intfname, addr))
+
+            node.rcmd('ip link set dev %s up' % intfname)
+            node.rcmd('ip link set dev %s mtu 1450' % intfname)
+            if not self.moveIntf(intfname, node):
+                raise Exception('interface move failed on node %s' % node)
+        return None  # May want to return something useful here
+
+
 # Some simple placement algorithms for MininetCluster
 
 class Placer( object ):
@@ -506,10 +599,10 @@ class Placer( object ):
 
 class RandomPlacer( Placer ):
     "Random placement"
-    def place( self, nodename ):
+    def place( self, node ):
         """Random placement function
-            nodename: node name"""
-        assert nodename  # please pylint
+            node: node"""
+        assert node  # please pylint
         # This may be slow with lots of servers
         return self.servers[ randrange( 0, len( self.servers ) ) ]
 
@@ -523,10 +616,10 @@ class RoundRobinPlacer( Placer ):
         Placer.__init__( self, *args, **kwargs )
         self.next = 0
 
-    def place( self, nodename ):
+    def place( self, node ):
         """Round-robin placement function
-            nodename: node name"""
-        assert nodename  # please pylint
+            node: node"""
+        assert node  # please pylint
         # This may be slow with lots of servers
         server = self.servers[ self.next ]
         self.next = ( self.next + 1 ) % len( self.servers )
@@ -564,7 +657,7 @@ class SwitchBinPlacer( Placer ):
         tickets = sum( [ binsizes[ server ] * [ server ]
                          for server in servers ], [] )
         # And assign one ticket to each node
-        return { node: ticket for node, ticket in zip( nodes, tickets ) }
+        return dict( zip( nodes, tickets ) )
 
     def calculatePlacement( self ):
         "Pre-calculate node placement"
@@ -621,21 +714,21 @@ class HostSwitchBinPlacer( Placer ):
         self.cset = frozenset( self.controllers )
         self.hind, self.sind, self.cind = 0, 0, 0
 
-    def place( self, nodename ):
+    def place( self, node ):
         """Simple placement algorithm:
             place nodes into evenly sized bins"""
         # Place nodes into bins
-        if nodename in self.hset:
+        if node in self.hset:
             server = self.servdict[ self.hind / self.hbin ]
             self.hind += 1
-        elif nodename in self.sset:
+        elif node in self.sset:
             server = self.servdict[ self.sind / self.sbin ]
             self.sind += 1
-        elif nodename in self.cset:
+        elif node in self.cset:
             server = self.servdict[ self.cind / self.cbin ]
             self.cind += 1
         else:
-            info( 'warning: unknown node', nodename )
+            info( 'warning: unknown node', node )
             server = self.servdict[ 0 ]
         return server
 
@@ -681,6 +774,7 @@ class MininetCluster( Mininet ):
         # Make sure control directory exists
         self.cdir = os.environ[ 'HOME' ] + '/.ssh/mn'
         errRun( [ 'mkdir', '-p', self.cdir ] )
+        # pylint: disable=unexpected-keyword-arg
         Mininet.__init__( self, *args, **params )
 
     def popen( self, cmd ):
@@ -757,16 +851,43 @@ class MininetCluster( Mininet ):
             if cfile:
                 config.setdefault( 'controlPath', cfile )
 
+    @staticmethod
+    def isLoopback( ipaddr ):
+        "Is ipaddr an IPv4 loopback address?"
+        return ipaddr.startswith( '127.' )
+
+    # pylint: disable=arguments-differ,signature-differs
     def addController( self, *args, **kwargs ):
         "Patch to update IP address to global IP address"
         controller = Mininet.addController( self, *args, **kwargs )
-        # Update IP address for controller that may not be local
-        if ( isinstance( controller, Controller)
-             and controller.IP() == '127.0.0.1'
-             and ' eth0:' in controller.cmd( 'ip link show' ) ):
-            Intf( 'eth0', node=controller ).updateIP()
+        controllerIP = controller.IP()
+        if ( not isinstance( controller, Controller ) or
+             not self.isLoopback( controller.IP() ) ):
+            return controller
+        # Find route to a different server IP address
+        serverIPs = [ ip for ip in self.serverIP.values()
+                      if ip != controllerIP ]
+        if not serverIPs:
+            return None  # no remote servers - loopback is fine
+        for remoteIP in serverIPs:
+            # Route should contain 'dev <intfname>'
+            route = controller.cmd( 'ip route get', remoteIP,
+                                    r'| egrep -o "dev\s[^[:space:]]+"' )
+            if not route:
+                raise Exception('addController: no route from', controller,
+                                'to', remoteIP )
+            intf = route.split()[ 1 ].strip()
+            if intf != 'lo':
+                break
+        if intf == 'lo':
+            raise Exception( 'addController: could not find external '
+                             'interface/IP for %s' % controller )
+        debug( 'adding', intf, 'to', controller )
+        Intf( intf, node=controller ).updateIP()
+        debug( controller, 'IP address updated to', controller.IP() )
         return controller
 
+    # pylint: disable=arguments-differ,signature-differs
     def buildFromTopo( self, *args, **kwargs ):
         "Start network"
         info( '*** Placing nodes\n' )
@@ -775,11 +896,14 @@ class MininetCluster( Mininet ):
         Mininet.buildFromTopo( self, *args, **kwargs )
 
 
-def testNsTunnels():
+# Default remote server for tests
+remoteServer = 'ubuntu2'
+
+def testNsTunnels( remote=remoteServer, link=RemoteGRELink ):
     "Test tunnels between nodes in namespaces"
-    net = Mininet( host=RemoteHost, link=RemoteLink )
-    h1 = net.addHost( 'h1' )
-    h2 = net.addHost( 'h2', server='ubuntu2' )
+    net = Mininet( host=RemoteHost, link=link, waitConnected=True )
+    h1 = net.addHost( 'h1')
+    h2 = net.addHost( 'h2', server=remote )
     net.addLink( h1, h2 )
     net.start()
     net.pingAll()
@@ -790,30 +914,31 @@ def testNsTunnels():
 # This shows how node options may be used to manage
 # cluster placement using the net.add*() API
 
-def testRemoteNet( remote='ubuntu2' ):
+def testRemoteNet( remote=remoteServer, link=RemoteGRELink ):
     "Test remote Node classes"
-    print '*** Remote Node Test'
+    info( '*** Remote Node Test\n' )
     net = Mininet( host=RemoteHost, switch=RemoteOVSSwitch,
-                   link=RemoteLink )
+                   link=link, controller=ClusterController,
+                   waitConnected=True )
     c0 = net.addController( 'c0' )
     # Make sure controller knows its non-loopback address
-    Intf( 'eth0', node=c0 ).updateIP()
-    print "*** Creating local h1"
+    info( "*** Creating local h1\n" )
     h1 = net.addHost( 'h1' )
-    print "*** Creating remote h2"
+    info( "*** Creating remote h2\n" )
     h2 = net.addHost( 'h2', server=remote )
-    print "*** Creating local s1"
+    info( "*** Creating local s1\n" )
     s1 = net.addSwitch( 's1' )
-    print "*** Creating remote s2"
+    info( "*** Creating remote s2\n" )
     s2 = net.addSwitch( 's2', server=remote )
-    print "*** Adding links"
+    info( "*** Adding links\n" )
     net.addLink( h1, s1 )
     net.addLink( s1, s2 )
     net.addLink( h2, s2 )
     net.start()
-    print 'Mininet is running on', quietRun( 'hostname' ).strip()
+    info( 'Mininet is running on', quietRun( 'hostname' ).strip(), '\n' )
     for node in c0, h1, h2, s1, s2:
-        print 'Node', node, 'is running on', node.cmd( 'hostname' ).strip()
+        info( 'Node', node, 'is running on',
+              node.cmd( 'hostname' ).strip(), '\n' )
     net.pingAll()
     CLI( net )
     net.stop()
@@ -828,7 +953,7 @@ def testRemoteNet( remote='ubuntu2' ):
 
 remoteHosts = [ 'h2' ]
 remoteSwitches = [ 's2' ]
-remoteServer = 'ubuntu2'
+
 
 def HostPlacer( name, *args, **params ):
     "Custom Host() constructor which places hosts on servers"
@@ -845,17 +970,28 @@ def SwitchPlacer( name, *args, **params ):
         return RemoteOVSSwitch( name, *args, **params )
 
 def ClusterController( *args, **kwargs):
-    "Custom Controller() constructor which updates its eth0 IP address"
+    "Custom Controller() constructor which updates its intf IP address"
+    intf = kwargs.pop( 'intf', '' )
     controller = Controller( *args, **kwargs )
     # Find out its IP address so that cluster switches can connect
-    Intf( 'eth0', node=controller ).updateIP()
+    if not intf:
+        output = controller.cmd(
+            r"ip a | egrep -o '\w+:\s\w+'" ).split( '\n' )
+        for line in output:
+            intf = line.split()[ -1 ]
+            if intf != 'lo':
+                break
+        if intf == 'lo':
+            raise Exception( 'Could not find non-loopback interface'
+                             'for %s' % controller )
+    Intf( intf, node=controller ).updateIP()
     return controller
 
-def testRemoteTopo():
+def testRemoteTopo( link=RemoteGRELink ):
     "Test remote Node classes using Mininet()/Topo() API"
     topo = LinearTopo( 2 )
     net = Mininet( topo=topo, host=HostPlacer, switch=SwitchPlacer,
-                   link=RemoteLink, controller=ClusterController )
+                   link=link, controller=ClusterController )
     net.start()
     net.pingAll()
     net.stop()
@@ -865,18 +1001,17 @@ def testRemoteTopo():
 # do random switch placement rather than completely random
 # host placement.
 
-def testRemoteSwitches():
+def testRemoteSwitches( remote=remoteServer, link=RemoteGRELink ):
     "Test with local hosts and remote switches"
-    servers = [ 'localhost', 'ubuntu2']
+    servers = [ 'localhost', remote]
     topo = TreeTopo( depth=4, fanout=2 )
-    net = MininetCluster( topo=topo, servers=servers,
+    net = MininetCluster( topo=topo, servers=servers, link=link,
                           placement=RoundRobinPlacer )
     net.start()
     net.pingAll()
     net.stop()
 
 
-#
 # For testing and demo purposes it would be nice to draw the
 # network graph and color it based on server.
 
@@ -884,31 +1019,35 @@ def testRemoteSwitches():
 # functions, for maximum ease of use. MininetCluster() also
 # pre-flights and multiplexes server connections.
 
-def testMininetCluster():
+def testMininetCluster( remote=remoteServer, link=RemoteGRELink ):
     "Test MininetCluster()"
-    servers = [ 'localhost', 'ubuntu2' ]
+    servers = [ 'localhost', remote ]
     topo = TreeTopo( depth=3, fanout=3 )
-    net = MininetCluster( topo=topo, servers=servers,
+    net = MininetCluster( topo=topo, servers=servers, link=link,
                           placement=SwitchBinPlacer )
     net.start()
     net.pingAll()
     net.stop()
 
-def signalTest():
+def signalTest( remote=remoteServer):
     "Make sure hosts are robust to signals"
-    h = RemoteHost( 'h0', server='ubuntu1' )
+    h = RemoteHost( 'h0', server=remote )
     h.shell.send_signal( SIGINT )
     h.shell.poll()
     if h.shell.returncode is None:
-        print 'OK: ', h, 'has not exited'
+        info( 'signalTest: SUCCESS: ', h, 'has not exited after SIGINT', '\n' )
     else:
-        print 'FAILURE:', h, 'exited with code', h.shell.returncode
+        info( 'signalTest: FAILURE:', h, 'exited with code',
+              h.shell.returncode, '\n' )
     h.stop()
 
+
 if __name__ == '__main__':
     setLogLevel( 'info' )
-    # testRemoteTopo()
-    # testRemoteNet()
-    # testMininetCluster()
-    # testRemoteSwitches()
-    signalTest()
+    remoteLink = RemoteSSHLink
+    testRemoteTopo(link=remoteLink)
+    testNsTunnels( remote=remoteServer, link=remoteLink )
+    testRemoteNet( remote=remoteServer, link=remoteLink)
+    testMininetCluster( remote=remoteServer, link=remoteLink)
+    testRemoteSwitches( remote=remoteServer, link=remoteLink)
+    signalTest( remote=remoteServer )
