@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 """
 Create a network where different switches are connected to
@@ -17,7 +17,7 @@ setLogLevel( 'info' )
 # Ignore the warning message that the remote isn't (yet) running
 c0 = Controller( 'c0', port=6633 )
 c1 = Controller( 'c1', port=6634 )
-c2 = RemoteController( 'c2', ip='127.0.0.1' )
+c2 = RemoteController( 'c2', ip='127.0.0.1', port=6633 )
 
 cmap = { 's1': c0, 's2': c1, 's3': c2 }
 
@@ -26,8 +26,9 @@ class MultiSwitch( OVSSwitch ):
     def start( self, controllers ):
         return OVSSwitch.start( self, [ cmap[ self.name ] ] )
 
+
 topo = TreeTopo( depth=2, fanout=2 )
-net = Mininet( topo=topo, switch=MultiSwitch, build=False )
+net = Mininet( topo=topo, switch=MultiSwitch, build=False, waitConnected=True )
 for c in [ c0, c1 ]:
     net.addController(c)
 net.build()
