@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 """
 This example creates a multi-controller network from semi-scratch by
@@ -11,51 +11,54 @@ Note that one could also create a custom switch class and pass it into
 the Mininet() constructor.
 """
 
+
 from mininet.net import Mininet
 from mininet.node import Controller, OVSSwitch
 from mininet.cli import CLI
-from mininet.log import setLogLevel
+from mininet.log import setLogLevel, info
 
 def multiControllerNet():
     "Create a network from semi-scratch with multiple controllers."
 
-    net = Mininet( controller=Controller, switch=OVSSwitch )
+    net = Mininet( controller=Controller, switch=OVSSwitch,
+                   waitConnected=True )
 
-    print "*** Creating (reference) controllers"
+    info( "*** Creating (reference) controllers\n" )
     c1 = net.addController( 'c1', port=6633 )
     c2 = net.addController( 'c2', port=6634 )
 
-    print "*** Creating switches"
+    info( "*** Creating switches\n" )
     s1 = net.addSwitch( 's1' )
     s2 = net.addSwitch( 's2' )
 
-    print "*** Creating hosts"
-    hosts1 = [ net.addHost( 'h%d' % n ) for n in 3, 4 ]
-    hosts2 = [ net.addHost( 'h%d' % n ) for n in 5, 6 ]
+    info( "*** Creating hosts\n" )
+    hosts1 = [ net.addHost( 'h%d' % n ) for n in ( 3, 4 ) ]
+    hosts2 = [ net.addHost( 'h%d' % n ) for n in ( 5, 6 ) ]
 
-    print "*** Creating links"
+    info( "*** Creating links\n" )
     for h in hosts1:
         net.addLink( s1, h )
     for h in hosts2:
         net.addLink( s2, h )
     net.addLink( s1, s2 )
 
-    print "*** Starting network"
+    info( "*** Starting network\n" )
     net.build()
     c1.start()
     c2.start()
     s1.start( [ c1 ] )
     s2.start( [ c2 ] )
 
-    print "*** Testing network"
+    info( "*** Testing network\n" )
     net.pingAll()
 
-    print "*** Running CLI"
+    info( "*** Running CLI\n" )
     CLI( net )
 
-    print "*** Stopping network"
+    info( "*** Stopping network\n" )
     net.stop()
 
+
 if __name__ == '__main__':
     setLogLevel( 'info' )  # for CLI output
     multiControllerNet()
