@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 """
 This example shows how to create a network and run multiple tests.
@@ -17,12 +17,14 @@ def ifconfigTest( net ):
     for host in hosts:
         info( host.cmd( 'ifconfig' ) )
 
+
 if __name__ == '__main__':
     lg.setLogLevel( 'info' )
     info( "*** Initializing Mininet and kernel modules\n" )
     OVSKernelSwitch.setup()
     info( "*** Creating network\n" )
-    network = Mininet( TreeTopo( depth=2, fanout=2 ), switch=OVSKernelSwitch )
+    network = Mininet( TreeTopo( depth=2, fanout=2), switch=OVSKernelSwitch,
+                       waitConnected=True )
     info( "*** Starting network\n" )
     network.start()
     info( "*** Running ping test\n" )
