@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 """
 This example monitors a number of hosts using host.popen() and
@@ -6,17 +6,14 @@ pmonitor()
 """
 
 from mininet.net import Mininet
-from mininet.node import CPULimitedHost
 from mininet.topo import SingleSwitchTopo
-from mininet.log import setLogLevel
-from mininet.util import custom, pmonitor
+from mininet.log import setLogLevel, info
+from mininet.util import pmonitor
 
-def monitorhosts( hosts=5, sched='cfs' ):
+def monitorhosts( hosts=5 ):
     "Start a bunch of pings and monitor them using popen"
     mytopo = SingleSwitchTopo( hosts )
-    cpu = .5 / hosts
-    myhost = custom( CPULimitedHost, cpu=cpu, sched=sched )
-    net = Mininet( topo=mytopo, host=myhost )
+    net = Mininet( topo=mytopo, waitConnected=True )
     net.start()
     # Start a bunch of pings
     popens = {}
@@ -27,10 +24,11 @@ def monitorhosts( hosts=5, sched='cfs' ):
     # Monitor them and print output
     for host, line in pmonitor( popens ):
         if host:
-            print "<%s>: %s" % ( host.name, line.strip() )
+            info( "<%s>: %s" % ( host.name, line ) )
     # Done
     net.stop()
 
+
 if __name__ == '__main__':
     setLogLevel( 'info' )
     monitorhosts( hosts=5 )
