@@ -5,7 +5,7 @@ Tests for controllers.py and controllers2.py
 """
 
 import unittest
-import pexpect
+from mininet.util import pexpect
 
 class testControllers( unittest.TestCase ):
 
