@@ -29,6 +29,8 @@ from subprocess import call
 from cmd import Cmd
 from os import isatty
 from select import poll, POLLIN
+import select
+import errno
 import sys
 import time
 import os
@@ -44,7 +46,8 @@ class CLI( Cmd ):
 
     prompt = 'mininet> '
 
-    def __init__( self, mininet, stdin=sys.stdin, script=None ):
+    def __init__( self, mininet, stdin=sys.stdin, script=None,
+                  **kwargs ):
         """Start and run interactive or batch mode CLI
            mininet: Mininet network object
            stdin: standard input for CLI
@@ -53,11 +56,10 @@ class CLI( Cmd ):
         # Local variable bindings for py command
         self.locals = { 'net': mininet }
         # Attempt to handle input
-        self.stdin = stdin
         self.inPoller = poll()
         self.inPoller.register( stdin )
         self.inputFile = script
-        Cmd.__init__( self )
+        Cmd.__init__( self, stdin=stdin, **kwargs )
         info( '*** Starting CLI:\n' )
 
         if self.inputFile:
@@ -77,6 +79,7 @@ class CLI( Cmd ):
             return
         cls.readlineInited = True
         try:
+            # pylint: disable=import-outside-toplevel
             from readline import ( read_history_file, write_history_file,
                                    set_history_length )
         except ImportError:
@@ -86,7 +89,15 @@ class CLI( Cmd ):
             if os.path.isfile( history_path ):
                 read_history_file( history_path )
                 set_history_length( 1000 )
-            atexit.register( lambda: write_history_file( history_path ) )
+
+            def writeHistory():
+                "Write out history file"
+                try:
+                    write_history_file( history_path )
+                except IOError:
+                    # Ignore probably spurious IOError
+                    pass
+            atexit.register( writeHistory )
 
     def run( self ):
         "Run our cmdloop(), catching KeyboardInterrupt"
@@ -139,10 +150,10 @@ class CLI( Cmd ):
         '  mininet> xterm h2\n\n'
     )
 
-    def do_help( self, line ):
+    def do_help( self, line ):  # pylint: disable=arguments-differ
         "Describe available CLI commands."
         Cmd.do_help( self, line )
-        if line is '':
+        if line == '':
             output( self.helpStr )
 
     def do_nodes( self, _line ):
@@ -171,6 +182,7 @@ class CLI( Cmd ):
         """Evaluate a Python expression.
            Node names may be used, e.g.: py h1.cmd('ls')"""
         try:
+            # pylint: disable=eval-used
             result = eval( line, globals(), self.getLocals() )
             if not result:
                 return
@@ -178,7 +190,7 @@ class CLI( Cmd ):
                 output( result + '\n' )
             else:
                 output( repr( result ) + '\n' )
-        except Exception, e:
+        except Exception as e:
             output( str( e ) + '\n' )
 
     # We are in fact using the exec() pseudo-function
@@ -189,7 +201,7 @@ class CLI( Cmd ):
             Node names may be used, e.g.: px print h1.cmd('ls')"""
         try:
             exec( line, globals(), self.getLocals() )
-        except Exception, e:
+        except Exception as e:
             output( str( e ) + '\n' )
 
     # pylint: enable=broad-except,exec-used
@@ -373,7 +385,7 @@ class CLI( Cmd ):
     def do_links( self, _line ):
         "Report on links"
         for link in self.mn.links:
-            print link, link.status()
+            output( link, link.status(), '\n' )
 
     def do_switch( self, line ):
         "Starts or stops a switch"
@@ -397,17 +409,22 @@ class CLI( Cmd ):
                 error( 'invalid command: '
                        'switch <switch name> {start, stop}\n' )
 
+    def do_wait( self, _line ):
+        "Wait until all switches have connected to a controller"
+        self.mn.waitConnected()
+
     def default( self, line ):
         """Called on an input line when the command prefix is not recognized.
-        Overridden to run shell commands when a node is the first CLI argument.
-        Past the first CLI argument, node names are automatically replaced with
-        corresponding IP addrs."""
+           Overridden to run shell commands when a node is the first
+           CLI argument.  Past the first CLI argument, node names are
+           automatically replaced with corresponding IP addrs."""
 
         first, args, line = self.parseline( line )
 
         if first in self.mn:
             if not args:
-                print "*** Enter a command for node: %s <cmd>" % first
+                error( '*** Please enter a command for node: %s <cmd>\n'
+                       % first )
                 return
             node = self.mn[ first ]
             rest = args.split( ' ' )
@@ -441,7 +458,7 @@ class CLI( Cmd ):
                 # XXX BL: this doesn't quite do what we want.
                 if False and self.inputFile:
                     key = self.inputFile.read( 1 )
-                    if key is not '':
+                    if key != '':
                         node.write( key )
                     else:
                         self.inputFile = None
@@ -458,6 +475,13 @@ class CLI( Cmd ):
                 # it's possible to interrupt ourselves after we've
                 # read data but before it has been printed.
                 node.sendInt()
+            except select.error as e:
+                # pylint: disable=unpacking-non-sequence
+                # pylint: disable=unbalanced-tuple-unpacking
+                errno_, errmsg = e.args
+                if errno_ != errno.EINTR:
+                    error( "select.error: %s, %s" % (errno_, errmsg) )
+                    node.sendInt()
 
     def precmd( self, line ):
         "allow for comments in the cli"
@@ -474,3 +498,4 @@ def isReadable( poller ):
         mask = fdmask[ 1 ]
         if mask & POLLIN:
             return True
+        return False
