@@ -57,16 +57,22 @@ import pty
 import re
 import signal
 import select
+from distutils.version import StrictVersion
+from re import findall
 from subprocess import Popen, PIPE
+from sys import exit  # pylint: disable=redefined-builtin
 from time import sleep
 
 from mininet.log import info, error, warn, debug
 from mininet.util import ( quietRun, errRun, errFail, moveIntf, isShellBuiltin,
-                           numCores, retry, mountCgroups )
+                           numCores, retry, mountCgroups, BaseString, decode,
+                           encode, getincrementaldecoder, Python3, which )
 from mininet.moduledeps import moduleDeps, pathCheck, TUN
 from mininet.link import Link, Intf, TCIntf, OVSIntf
-from re import findall
-from distutils.version import StrictVersion
+
+
+# pylint: disable=too-many-arguments
+
 
 class Node( object ):
     """A virtual network node is simply a shell in a network namespace.
@@ -87,12 +93,19 @@ class Node( object ):
         self.privateDirs = params.get( 'privateDirs', [] )
         self.inNamespace = params.get( 'inNamespace', inNamespace )
 
+        # Python 3 complains if we don't wait for shell exit
+        self.waitExited = params.get( 'waitExited', Python3 )
+
         # Stash configuration parameters for future reference
         self.params = params
 
-        self.intfs = {}  # dict of port numbers to interfaces
-        self.ports = {}  # dict of interfaces to port numbers
-                         # replace with Port objects, eventually ?
+        # dict of port numbers to interfacse
+        self.intfs = {}
+
+        # dict of interfaces to port numbers
+        # todo: replace with Port objects, eventually ?
+        self.ports = {}
+
         self.nameToIntf = {}  # dict of interface names to Intfs
 
         # Make pylint happy
@@ -102,7 +115,11 @@ class Node( object ):
         self.waiting = False
         self.readbuf = ''
 
+        # Incremental decoder for buffered reading
+        self.decoder = getincrementaldecoder()
+
         # Start command interpreter shell
+        self.master, self.slave = None, None  # pylint
         self.startShell()
         self.mountPrivateDirs()
 
@@ -135,14 +152,18 @@ class Node( object ):
         # -s: pass $* to shell, and make process easy to find in ps
         # prompt is set to sentinel chr( 127 )
         cmd = [ 'mnexec', opts, 'env', 'PS1=' + chr( 127 ),
-                'bash', '--norc', '-is', 'mininet:' + self.name ]
+                'bash', '--norc', '--noediting',
+                '-is', 'mininet:' + self.name ]
+
         # Spawn a shell subprocess in a pseudo-tty, to disable buffering
         # in the subprocess and insulate it from signals (e.g. SIGINT)
         # received by the parent
-        master, slave = pty.openpty()
-        self.shell = self._popen( cmd, stdin=slave, stdout=slave, stderr=slave,
-                                  close_fds=False )
-        self.stdin = os.fdopen( master, 'rw' )
+        self.master, self.slave = pty.openpty()
+        self.shell = self._popen( cmd, stdin=self.slave, stdout=self.slave,
+                                  stderr=self.slave, close_fds=False )
+        # XXX BL: This doesn't seem right, and we should also probably
+        # close our files when we exit...
+        self.stdin = os.fdopen( self.master, 'r' )
         self.stdout = self.stdin
         self.pid = self.shell.pid
         self.pollOut = select.poll()
@@ -169,7 +190,7 @@ class Node( object ):
     def mountPrivateDirs( self ):
         "mount private directories"
         # Avoid expanding a string into a list of chars
-        assert not isinstance( self.privateDirs, basestring )
+        assert not isinstance( self.privateDirs, BaseString )
         for directory in self.privateDirs:
             if isinstance( directory, tuple ):
                 # mount given private directory
@@ -198,7 +219,9 @@ class Node( object ):
             params: parameters to Popen()"""
         # Leave this is as an instance method for now
         assert self
-        return Popen( cmd, **params )
+        popen = Popen( cmd, **params )
+        debug( '_popen', cmd, popen.pid )
+        return popen
 
     def cleanup( self ):
         "Help python collect its garbage."
@@ -207,23 +230,30 @@ class Node( object ):
         # for intfName in self.intfNames():
         # if self.name in intfName:
         # quietRun( 'ip link del ' + intfName )
+        if self.shell:
+            # Close ptys
+            self.stdin.close()
+            os.close(self.slave)
+            if self.waitExited:
+                debug( 'waiting for', self.pid, 'to terminate\n' )
+                self.shell.wait()
         self.shell = None
 
     # Subshell I/O, commands and control
 
-    def read( self, maxbytes=1024 ):
+    def read( self, size=1024 ):
         """Buffered read from node, potentially blocking.
-           maxbytes: maximum number of bytes to return"""
+           size: maximum number of characters to return"""
         count = len( self.readbuf )
-        if count < maxbytes:
-            data = os.read( self.stdout.fileno(), maxbytes - count )
-            self.readbuf += data
-        if maxbytes >= len( self.readbuf ):
+        if count < size:
+            data = os.read( self.stdout.fileno(), size - count )
+            self.readbuf += self.decoder.decode( data )
+        if size >= len( self.readbuf ):
             result = self.readbuf
             self.readbuf = ''
         else:
-            result = self.readbuf[ :maxbytes ]
-            self.readbuf = self.readbuf[ maxbytes: ]
+            result = self.readbuf[ :size ]
+            self.readbuf = self.readbuf[ size: ]
         return result
 
     def readline( self ):
@@ -240,7 +270,7 @@ class Node( object ):
     def write( self, data ):
         """Write data to node.
            data: string"""
-        os.write( self.stdin.fileno(), data )
+        os.write( self.stdin.fileno(), encode( data ) )
 
     def terminate( self ):
         "Send kill signal to Node and clean up after it."
@@ -263,6 +293,7 @@ class Node( object ):
            returns: result of poll()"""
         if len( self.readbuf ) == 0:
             return self.pollOut.poll( timeoutms )
+        return None
 
     def sendCmd( self, *args, **kwargs ):
         """Send a command, followed by a command to echo a sentinel,
@@ -356,6 +387,7 @@ class Node( object ):
             return self.waitOutput( verbose )
         else:
             warn( '(%s exited - ignoring cmd%s)\n' % ( self, args ) )
+        return None
 
     def cmdPrint( self, *args):
         """Call cmd and printing its output
@@ -370,23 +402,23 @@ class Node( object ):
                      'mncmd':
                      [ 'mnexec', '-da', str( self.pid ) ] }
         defaults.update( kwargs )
+        shell = defaults.pop( 'shell', False )
         if len( args ) == 1:
             if isinstance( args[ 0 ], list ):
                 # popen([cmd, arg1, arg2...])
                 cmd = args[ 0 ]
-            elif isinstance( args[ 0 ], basestring ):
+            elif isinstance( args[ 0 ], BaseString ):
                 # popen("cmd arg1 arg2...")
-                cmd = args[ 0 ].split()
+                cmd = [ args[ 0 ] ] if shell else args[ 0 ].split()
             else:
                 raise Exception( 'popen() requires a string or list' )
         elif len( args ) > 0:
             # popen( cmd, arg1, arg2... )
             cmd = list( args )
+        if shell:
+            cmd = [ os.environ[ 'SHELL' ], '-c' ] + [ ' '.join( cmd ) ]
         # Attach to our namespace  using mnexec -a
         cmd = defaults.pop( 'mncmd' ) + cmd
-        # Shell requires a string, not a list!
-        if defaults.get( 'shell', False ):
-            cmd = ' '.join( cmd )
         popen = self._popen( cmd, **defaults )
         return popen
 
@@ -398,7 +430,7 @@ class Node( object ):
         # Warning: this can fail with large numbers of fds!
         out, err = popen.communicate()
         exitcode = popen.wait()
-        return out, err, exitcode
+        return decode( out ), decode( err ), exitcode
 
     # Interface management, configuration, and routing
 
@@ -431,6 +463,15 @@ class Node( object ):
             debug( 'moving', intf, 'into namespace for', self.name, '\n' )
             moveIntfFn( intf.name, self  )
 
+    def delIntf( self, intf ):
+        """Remove interface from Node's known interfaces
+           Note: to fully delete interface, call intf.delete() instead"""
+        port = self.ports.get( intf )
+        if port is not None:
+            del self.intfs[ port ]
+            del self.ports[ intf ]
+            del self.nameToIntf[ intf.name ]
+
     def defaultIntf( self ):
         "Return interface for lowest port"
         ports = self.intfs.keys()
@@ -439,6 +480,7 @@ class Node( object ):
         else:
             warn( '*** defaultIntf: warning:', self.name,
                   'has no interfaces\n' )
+        return None
 
     def intf( self, intf=None ):
         """Return our interface object with given string name,
@@ -451,7 +493,7 @@ class Node( object ):
         """
         if not intf:
             return self.defaultIntf()
-        elif isinstance( intf, basestring):
+        elif isinstance( intf, BaseString):
             return self.nameToIntf[ intf ]
         else:
             return intf
@@ -478,7 +520,7 @@ class Node( object ):
         # explicitly so that we won't get errors if we run before they
         # have been removed by the kernel. Unfortunately this is very slow,
         # at least with Linux kernels before 2.6.33
-        for intf in self.intfs.values():
+        for intf in list( self.intfs.values() ):
             # Protect against deleting hardware interfaces
             if ( self.name in intf.name ) or ( not checkName ):
                 intf.delete()
@@ -503,7 +545,7 @@ class Node( object ):
         """Set the default route to go through intf.
            intf: Intf or {dev <intfname> via <gw-ip> ...}"""
         # Note setParam won't call us if intf is none
-        if isinstance( intf, basestring ) and ' ' in intf:
+        if isinstance( intf, BaseString ) and ' ' in intf:
             params = intf
         else:
             params = 'dev %s' % intf
@@ -550,12 +592,12 @@ class Node( object ):
            method: config method name
            param: arg=value (ignore if value=None)
            value may also be list or dict"""
-        name, value = param.items()[ 0 ]
+        name, value = list( param.items() )[ 0 ]
         if value is None:
-            return
+            return None
         f = getattr( self, method, None )
         if not f:
-            return
+            return None
         if isinstance( value, list ):
             result = f( *value )
         elif isinstance( value, dict ):
@@ -599,7 +641,7 @@ class Node( object ):
 
     def intfList( self ):
         "List of our interfaces sorted by port number"
-        return [ self.intfs[ p ] for p in sorted( self.intfs.iterkeys() ) ]
+        return [ self.intfs[ p ] for p in sorted( self.intfs.keys() ) ]
 
     def intfNames( self ):
         "The names of our interfaces sorted by port number"
@@ -623,11 +665,12 @@ class Node( object ):
     @classmethod
     def checkSetup( cls ):
         "Make sure our class and superclasses are set up"
-        while cls and not getattr( cls, 'isSetup', True ):
-            cls.setup()
-            cls.isSetup = True
+        clas = cls
+        while clas and not getattr( clas, 'isSetup', True ):
+            clas.setup()
+            clas.isSetup = True
             # Make pylint happy
-            cls = getattr( type( cls ), '__base__', None )
+            clas = getattr( type( clas ), '__base__', None )
 
     @classmethod
     def setup( cls ):
@@ -693,8 +736,8 @@ class CPULimitedHost( Host ):
            args: Popen() args, single list, or string
            kwargs: Popen() keyword args"""
         # Tell mnexec to execute command in our cgroup
-        mncmd = [ 'mnexec', '-g', self.name,
-                  '-da', str( self.pid ) ]
+        mncmd = kwargs.pop( 'mncmd', [ 'mnexec', '-g', self.name,
+                                       '-da', str( self.pid ) ] )
         # if our cgroup is not given any cpu time,
         # we cannot assign the RR Scheduler.
         if self.sched == 'rt':
@@ -808,6 +851,7 @@ class CPULimitedHost( Host ):
         errFail( 'cgclassify -g cpuset:/%s %s' % (
                  self.name, self.pid ) )
 
+    # pylint: disable=arguments-differ
     def config( self, cpu=-1, cores=None, **params ):
         """cpu: desired overall system CPU fraction
            cores: (real) core(s) this host can run on
@@ -870,7 +914,7 @@ class Switch( Node ):
         "Return correctly formatted dpid from dpid or switch name (s1 -> 1)"
         if dpid:
             # Remove any colons and make sure it's a good hex number
-            dpid = dpid.translate( None, ':' )
+            dpid = dpid.replace( ':', '' )
             assert len( dpid ) <= self.dpidLen and int( dpid, 16 ) >= 0
         else:
             # Use hex of the first number in the switch name
@@ -878,6 +922,7 @@ class Switch( Node ):
             if nums:
                 dpid = hex( int( nums[ 0 ] ) )[ 2: ]
             else:
+                self.terminate()  # Python 3.6 crash workaround
                 raise Exception( 'Unable to derive default datapath ID - '
                                  'please either specify a dpid or use a '
                                  'canonical switch name such as s23.' )
@@ -899,6 +944,7 @@ class Switch( Node ):
         else:
             error( '*** Error: %s has execed and cannot accept commands' %
                    self.name )
+        return None
 
     def connected( self ):
         "Is the switch connected to a controller? (override this method)"
@@ -1084,6 +1130,7 @@ class OVSSwitch( Switch ):
         if self.batch:
             cmd = ' '.join( str( arg ).strip() for arg in args )
             self.commands.append( cmd )
+            return None
         else:
             return self.cmd( 'ovs-vsctl', *args, **kwargs )
 
@@ -1151,6 +1198,7 @@ class OVSSwitch( Switch ):
             opts += ' protocols=%s' % self.protocols
         if self.stp and self.failMode == 'standalone':
             opts += ' stp_enable=true'
+        opts += ' other-config:dp-desc=%s' % self.name
         return opts
 
     def start( self, controllers ):
@@ -1220,7 +1268,7 @@ class OVSSwitch( Switch ):
             run( cmds, shell=True )
         # Reapply link config if necessary...
         for switch in switches:
-            for intf in switch.intfs.itervalues():
+            for intf in switch.intfs.values():
                 if isinstance( intf, TCIntf ):
                     intf.config( **intf.params )
         return switches
@@ -1246,7 +1294,7 @@ class OVSSwitch( Switch ):
         pids = ' '.join( str( switch.pid ) for switch in switches )
         run( 'kill -HUP ' + pids )
         for switch in switches:
-            switch.shell = None
+            switch.terminate()
         return switches
 
 
@@ -1270,7 +1318,7 @@ class OVSBridge( OVSSwitch ):
         "Are we forwarding yet?"
         if self.stp:
             status = self.dpctl( 'show' )
-            return 'STP_FORWARD' in status and not 'STP_LEARN' in status
+            return 'STP_FORWARD' in status and 'STP_LEARN' not in status
         else:
             return True
 
@@ -1350,10 +1398,12 @@ class Controller( Node ):
        OpenFlow controller."""
 
     def __init__( self, name, inNamespace=False, command='controller',
-                  cargs='-v ptcp:%d', cdir=None, ip="127.0.0.1",
-                  port=6653, protocol='tcp', **params ):
+                  cargs='ptcp:%d', cdir=None, ip="127.0.0.1",
+                  port=6653, protocol='tcp', verbose=False, **params ):
         self.command = command
         self.cargs = cargs
+        if verbose:
+            cargs = '-v ' + cargs
         self.cdir = cdir
         # Accept 'ip:port' syntax as shorthand
         if ':' in ip:
@@ -1395,6 +1445,7 @@ class Controller( Node ):
                   ' 1>' + cout + ' 2>' + cout + ' &' )
         self.execed = False
 
+    # pylint: disable=arguments-differ,signature-differs
     def stop( self, *args, **kwargs ):
         "Stop controller."
         self.cmd( 'kill %' + self.command )
@@ -1418,21 +1469,21 @@ class Controller( Node ):
     @classmethod
     def isAvailable( cls ):
         "Is controller available?"
-        return quietRun( 'which controller' )
+        return which( 'controller' )
 
 
 class OVSController( Controller ):
     "Open vSwitch controller"
-    def __init__( self, name, command='ovs-controller', **kwargs ):
-        if quietRun( 'which test-controller' ):
-            command = 'test-controller'
-        Controller.__init__( self, name, command=command, **kwargs )
+    def __init__( self, name, **kwargs ):
+        kwargs.setdefault( 'command', self.isAvailable() or
+                           'ovs-controller' )
+        Controller.__init__( self, name, **kwargs )
 
     @classmethod
     def isAvailable( cls ):
-        return ( quietRun( 'which ovs-controller' ) or
-                 quietRun( 'which test-controller' ) or
-                 quietRun( 'which ovs-testcontroller' ) )
+        return (which( 'ovs-controller' ) or
+                which( 'test-controller' ) or
+                which( 'ovs-testcontroller' ))
 
 class NOX( Controller ):
     "Controller to run a NOX application."
@@ -1445,7 +1496,7 @@ class NOX( Controller ):
             warn( 'warning: no NOX modules specified; '
                   'running packetdump only\n' )
             noxArgs = [ 'packetdump' ]
-        elif type( noxArgs ) not in ( list, tuple ):
+        elif not isinstance( noxArgs, ( list, tuple ) ):
             noxArgs = [ noxArgs ]
 
         if 'NOX_CORE_DIR' not in os.environ:
@@ -1454,32 +1505,26 @@ class NOX( Controller ):
 
         Controller.__init__( self, name,
                              command=noxCoreDir + '/nox_core',
-                             cargs='--libdir=/usr/local/lib -v -i ptcp:%s ' +
+                             cargs='--libdir=/usr/local/lib -v '
+                             '-i ptcp:%s ' +
                              ' '.join( noxArgs ),
                              cdir=noxCoreDir,
                              **kwargs )
 
 class Ryu( Controller ):
-    "Controller to run Ryu application"
-    def __init__( self, name, *ryuArgs, **kwargs ):
+    "Ryu OpenFlow Controller"
+    def __init__( self, name, ryuArgs='ryu.app.simple_switch',
+                  command='ryu run', **kwargs ):
         """Init.
-        name: name to give controller.
-        ryuArgs: arguments and modules to pass to Ryu"""
-        homeDir = quietRun( 'printenv HOME' ).strip( '\r\n' )
-        ryuCoreDir = '%s/ryu/ryu/app/' % homeDir
-        if not ryuArgs:
-            warn( 'warning: no Ryu modules specified; '
-                  'running simple_switch only\n' )
-            ryuArgs = [ ryuCoreDir + 'simple_switch.py' ]
-        elif type( ryuArgs ) not in ( list, tuple ):
-            ryuArgs = [ ryuArgs ]
-
-        Controller.__init__( self, name,
-                             command='ryu-manager',
-                             cargs='--ofp-tcp-listen-port %s ' +
-                             ' '.join( ryuArgs ),
-                             cdir=ryuCoreDir,
-                             **kwargs )
+           name: name to give controller.
+           ryuArgs: modules to pass to Ryu (ryu.app.simple_switch)
+           command: comand to run Ryu ('ryu run')"""
+        if isinstance( ryuArgs, ( list, tuple ) ):
+            ryuArgs = ' '.join( ryuArgs )
+        cargs = kwargs.pop(
+            'cargs', ryuArgs + ' --ofp-tcp-listen-port %s' )
+        Controller.__init__( self, name, command=command,
+                             cargs=cargs, **kwargs )
 
 
 class RemoteController( Controller ):
@@ -1498,6 +1543,7 @@ class RemoteController( Controller ):
         "Overridden to do nothing."
         return
 
+    # pylint: disable=arguments-differ
     def stop( self ):
         "Overridden to do nothing."
         return
@@ -1529,6 +1575,7 @@ class RemoteController( Controller ):
         else:
             return True
 
+
 DefaultControllers = ( Controller, OVSController )
 
 def findController( controllers=DefaultControllers ):
@@ -1536,6 +1583,7 @@ def findController( controllers=DefaultControllers ):
     for controller in controllers:
         if controller.isAvailable():
             return controller
+    return None
 
 def DefaultController( name, controllers=DefaultControllers, **kwargs ):
     "Find a controller that is available and instantiate it"
