@@ -21,7 +21,10 @@ def runTests( testDir, verbosity=1 ):
     # discover all tests in testDir
     testSuite = defaultTestLoader.discover( testDir )
     # run tests
-    TextTestRunner( verbosity=verbosity ).run( testSuite )
+    success = ( TextTestRunner( verbosity=verbosity )
+                .run( testSuite ).wasSuccessful() )
+    sys.exit( 0 if success else 1 )
+
 
 if __name__ == '__main__':
     setLogLevel( 'warning' )
