@@ -6,13 +6,16 @@ Tests for the Mininet Walkthrough
 TODO: missing xterm test
 """
 
-import unittest
-import pexpect
 import os
 import re
-from mininet.util import quietRun
+import unittest
+
 from distutils.version import StrictVersion
-from time import sleep
+from sys import stdout
+
+from mininet.util import quietRun, pexpect
+from mininet.clean import cleanup
+
 
 def tsharkVersion():
     "Return tshark version"
@@ -28,6 +31,11 @@ class testWalkthrough( unittest.TestCase ):
 
     prompt = 'mininet>'
 
+    @staticmethod
+    def setup():
+        "Be paranoid and run cleanup() before each test"
+        cleanup()
+
     # PART 1
     def testHelp( self ):
         "Check the usage message"
@@ -43,17 +51,18 @@ class testWalkthrough( unittest.TestCase ):
             tshark = pexpect.spawn( 'tshark -i lo -R of' )
         else:
             tshark = pexpect.spawn( 'tshark -i lo -Y openflow_v1' )
-        tshark.expect( [ 'Capturing on lo', "Capturing on 'Loopback'" ] )
+        tshark.expect( [ 'Capturing on lo', "Capturing on 'Loopback" ] )
         mn = pexpect.spawn( 'mn --test pingall' )
         mn.expect( '0% dropped' )
         tshark.expect( [ '74 Hello', '74 of_hello', '74 Type: OFPT_HELLO' ] )
         tshark.sendintr()
         mn.expect( pexpect.EOF )
+        tshark.expect( 'aptured' )  # 'xx packets captured'
         tshark.expect( pexpect.EOF )
 
     def testBasic( self ):
         "Test basic CLI commands (help, nodes, net, dump)"
-        p = pexpect.spawn( 'mn' )
+        p = pexpect.spawn( 'mn -w' )
         p.expect( self.prompt )
         # help command
         p.sendline( 'help' )
@@ -67,7 +76,7 @@ class testWalkthrough( unittest.TestCase ):
         p.expect( self.prompt )
         # net command
         p.sendline( 'net' )
-        expected = [ x for x in nodes ]
+        expected = list( nodes )
         while len( expected ) > 0:
             index = p.expect( expected )
             node = p.match.group( 0 )
@@ -92,17 +101,18 @@ class testWalkthrough( unittest.TestCase ):
 
     def testHostCommands( self ):
         "Test ifconfig and ps on h1 and s1"
-        p = pexpect.spawn( 'mn' )
+        p = pexpect.spawn( 'mn -w' )
         p.expect( self.prompt )
         # Third pattern is a local interface beginning with 'eth' or 'en'
-        interfaces = [ 'h1-eth0', 's1-eth1', r'[^-](eth|en)\w*\d', 'lo',
+        interfaces = [ r'h1-eth0[:\s]', r's1-eth1[:\s]',
+                       r'[^-](eth|en)\w*\d[:\s]', r'lo[:\s]',
                        self.prompt ]
         # h1 ifconfig
         p.sendline( 'h1 ifconfig -a' )
         ifcount = 0
         while True:
             index = p.expect( interfaces )
-            if index == 0 or index == 3:
+            if index in (0, 3):
                 ifcount += 1
             elif index == 1:
                 self.fail( 's1 interface displayed in "h1 ifconfig"' )
@@ -118,11 +128,11 @@ class testWalkthrough( unittest.TestCase ):
             index = p.expect( interfaces )
             if index == 0:
                 self.fail( 'h1 interface displayed in "s1 ifconfig"' )
-            elif index == 1 or index == 2 or index == 3:
+            elif index in (1, 2, 3):
                 ifcount += 1
             else:
                 break
-        self.assertTrue( ifcount >= 3, 'Missing interfaces on s1')
+        self.assertTrue( ifcount <= 3, 'Missing interfaces on s1')
         # h1 ps
         p.sendline( "h1 ps -a | egrep -v 'ps|grep'" )
         p.expect( self.prompt )
@@ -143,7 +153,7 @@ class testWalkthrough( unittest.TestCase ):
 
     def testConnectivity( self ):
         "Test ping and pingall"
-        p = pexpect.spawn( 'mn' )
+        p = pexpect.spawn( 'mn -w' )
         p.expect( self.prompt )
         p.sendline( 'h1 ping -c 1 h2' )
         p.expect( '1 packets transmitted, 1 received' )
@@ -156,14 +166,20 @@ class testWalkthrough( unittest.TestCase ):
 
     def testSimpleHTTP( self ):
         "Start an HTTP server on h1 and wget from h2"
-        p = pexpect.spawn( 'mn' )
+        if 'Python 2' in quietRun( 'python --version' ):
+            httpserver = 'SimpleHTTPServer'
+        else:
+            httpserver = 'http.server'
+        p = pexpect.spawn( 'mn -w', logfile=stdout )
+        p.expect( self.prompt )
+        p.sendline( 'h1 python -m %s 80 >& /dev/null &' % httpserver )
         p.expect( self.prompt )
-        p.sendline( 'h1 python -m SimpleHTTPServer 80 &' )
         # The walkthrough doesn't specify a delay here, and
         # we also don't read the output (also a possible problem),
-        # but for now let's wait a couple of seconds to make
+        # but for now let's wait a number of seconds to make
         # it less likely to fail due to the race condition.
-        sleep( 2 )
+        p.sendline( 'px from mininet.util import waitListening;'
+                    'waitListening(h1, port=80, timeout=30)' )
         p.expect( self.prompt )
         p.sendline( ' h2 wget -O - h1' )
         p.expect( '200 OK' )
@@ -208,22 +224,24 @@ class testWalkthrough( unittest.TestCase ):
 
     def testLinkChange( self ):
         "Test TCLink bw and delay"
-        p = pexpect.spawn( 'mn --link tc,bw=10,delay=10ms' )
+        p = pexpect.spawn( 'mn -w --link tc,bw=10,delay=10ms' )
+        p.expect( self.prompt )
+        p.sendline( 'h1 route && ping -c1 h2' )
         # test bw
         p.expect( self.prompt )
         p.sendline( 'iperf' )
         p.expect( r"Results: \['([\d\.]+) Mbits/sec'," )
         bw = float( p.match.group( 1 ) )
-        self.assertTrue( bw < 10.1, 'Bandwidth > 10 Mb/s')
-        self.assertTrue( bw > 9.0, 'Bandwidth < 9 Mb/s')
+        self.assertTrue( bw < 10.1, 'Bandwidth %.2f >= 10.1 Mb/s' % bw )
+        self.assertTrue( bw > 9.0, 'Bandwidth %.2f <= 9 Mb/s' % bw )
         p.expect( self.prompt )
         # test delay
         p.sendline( 'h1 ping -c 4 h2' )
         p.expect( r'rtt min/avg/max/mdev = '
                   r'([\d\.]+)/([\d\.]+)/([\d\.]+)/([\d\.]+) ms' )
         delay = float( p.match.group( 2 ) )
-        self.assertTrue( delay > 40, 'Delay < 40ms' )
-        self.assertTrue( delay < 45, 'Delay > 40ms' )
+        self.assertTrue( delay >= 40, 'Delay < 40ms' )
+        self.assertTrue( delay <= 50, 'Delay > 50ms' )
         p.expect( self.prompt )
         p.sendline( 'exit' )
         p.wait()
@@ -260,7 +278,7 @@ class testWalkthrough( unittest.TestCase ):
         p.expect( self.prompt )
         for i in range( 1, 3 ):
             p.sendline( 'h%d ifconfig' % i )
-            p.expect( 'HWaddr 00:00:00:00:00:0%d' % i )
+            p.expect( r'\s00:00:00:00:00:0%d\s' % i )
             p.expect( self.prompt )
         p.sendline( 'exit' )
         p.expect( pexpect.EOF )
@@ -286,12 +304,14 @@ class testWalkthrough( unittest.TestCase ):
         "Test running user switch in its own namespace"
         p = pexpect.spawn( 'mn --innamespace --switch user' )
         p.expect( self.prompt )
-        interfaces = [ 'h1-eth0', 's1-eth1', '[^-]eth0', 'lo', self.prompt ]
+        interfaces = [ r'h1-eth0[:\s]', r's1-eth1[:\s]',
+                       r'[^-](eth|en)\w*\d[:\s]', r'lo[:\s]',
+                       self.prompt ]
         p.sendline( 's1 ifconfig -a' )
         ifcount = 0
         while True:
             index = p.expect( interfaces )
-            if index == 1 or index == 3:
+            if index in (1, 3):
                 ifcount += 1
             elif index == 0:
                 self.fail( 'h1 interface displayed in "s1 ifconfig"' )
@@ -312,7 +332,7 @@ class testWalkthrough( unittest.TestCase ):
     # PART 3
     def testPythonInterpreter( self ):
         "Test py and px by checking IP for h1 and adding h3"
-        p = pexpect.spawn( 'mn' )
+        p = pexpect.spawn( 'mn -w' )
         p.expect( self.prompt )
         # test host IP
         p.sendline( 'py h1.IP()' )
@@ -334,7 +354,7 @@ class testWalkthrough( unittest.TestCase ):
 
     def testLink( self ):
         "Test link CLI command using ping"
-        p = pexpect.spawn( 'mn' )
+        p = pexpect.spawn( 'mn -w' )
         p.expect( self.prompt )
         p.sendline( 'link s1 h1 down' )
         p.expect( self.prompt )
