@@ -13,6 +13,9 @@ setup for testing, and can even be emulated with the Mininet package.
 
 from mininet.util import irange, natural, naturalSeq
 
+# pylint: disable=too-many-arguments
+
+
 class MultiGraph( object ):
     "Utility class to track nodes and edges - replaces networkx.MultiGraph"
 
@@ -34,7 +37,7 @@ class MultiGraph( object ):
            key: optional key
            attr_dict: optional attribute dict
            attrs: more attributes
-           warning: udpates attr_dict with attrs"""
+           warning: updates attr_dict with attrs"""
         attr_dict = {} if attr_dict is None else attr_dict
         attr_dict.update( attrs )
         self.node.setdefault( src, {} )
@@ -56,13 +59,13 @@ class MultiGraph( object ):
         return self.node.items() if data else self.node.keys()
 
     def edges_iter( self, data=False, keys=False ):
-        "Iterator: return graph edges"
-        for src, entry in self.edge.iteritems():
-            for dst, keys in entry.iteritems():
+        "Iterator: return graph edges, optionally with data and keys"
+        for src, entry in self.edge.items():
+            for dst, entrykeys in entry.items():
                 if src > dst:
                     # Skip duplicate edges
                     continue
-                for k, attrs in keys.iteritems():
+                for k, attrs in entrykeys.items():
                     if data:
                         if keys:
                             yield( src, dst, k, attrs )
@@ -156,8 +159,7 @@ class Topo( object ):
         port1, port2 = self.addPort( node1, node2, port1, port2 )
         opts = dict( opts )
         opts.update( node1=node1, node2=node2, port1=port1, port2=port2 )
-        self.g.add_edge(node1, node2, key, opts )
-        return key
+        return self.g.add_edge(node1, node2, key, opts )
 
     def nodes( self, sort=True ):
         "Return nodes in graph"
